//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/coregui/DataLoaders/AbstractDataLoader1D.h
//! @brief     Defines class AbstractDataLoader1D
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_COREGUI_DATALOADERS_ABSTRACTDATALOADER1D_H
#define BORNAGAIN_GUI_COREGUI_DATALOADERS_ABSTRACTDATALOADER1D_H

#include "GUI/coregui/DataLoaders/AbstractDataLoader.h"

//! Base class for data loaders for 1D import

class AbstractDataLoader1D : public AbstractDataLoader {
};

#endif // BORNAGAIN_GUI_COREGUI_DATALOADERS_ABSTRACTDATALOADER1D_H
