//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      gui2/resources/resources.h
//! @brief     Defines class CLASS?
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2020
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI2_RESOURCES_RESOURCES_H
#define BORNAGAIN_GUI2_RESOURCES_RESOURCES_H

#include <QtGlobal>

inline void InitIconResources()
{
    Q_INIT_RESOURCE(icons);
}

#endif // BORNAGAIN_GUI2_RESOURCES_RESOURCES_H
