/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Designer of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

// Patched JWu 12nov20 because QFlags(0) was deprecated

#ifndef BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOX_H
#define BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOX_H

#include "GUI/coregui/Views/widgetbox/qdesigner_widgetbox_p.h"
#include "GUI/coregui/Views/widgetbox/widgetbox_global.h"

QT_BEGIN_NAMESPACE

// class QDesignerFormEditorInterface;
class QDesignerFormWindowInterface;

class SampleDesignerInterface;

namespace qdesigner_internal {

class WidgetBoxTreeWidget;

class QT_WIDGETBOX_EXPORT WidgetBox : public QDesignerWidgetBox {
    Q_OBJECT
public:
    //    explicit WidgetBox(QDesignerFormEditorInterface *core, QWidget *parent = 0,
    //    Qt::WindowFlags flags = 0);
    explicit WidgetBox(SampleDesignerInterface* core, QWidget* parent = nullptr,
                       Qt::WindowFlags flags = {});
    virtual ~WidgetBox();

    //    QDesignerFormEditorInterface *core() const;
    SampleDesignerInterface* core() const;

    virtual int categoryCount() const;
    virtual Category category(int cat_idx) const;
    virtual void addCategory(const Category& cat);
    virtual void removeCategory(int cat_idx);

    virtual int widgetCount(int cat_idx) const;
    virtual Widget widget(int cat_idx, int wgt_idx) const;
    virtual void addWidget(int cat_idx, const Widget& wgt);
    virtual void removeWidget(int cat_idx, int wgt_idx);

    void dropWidgets(const QList<QDesignerDnDItemInterface*>& item_list,
                     const QPoint& global_mouse_pos);

    virtual void setFileName(const QString& file_name);
    virtual QString fileName() const;
    virtual bool load();
    virtual bool save();

    virtual bool loadContents(const QString& contents);
    virtual QIcon iconForWidget(const QString& className, const QString& category = "") const;

protected:
    virtual void dragEnterEvent(QDragEnterEvent* event);
    virtual void dragMoveEvent(QDragMoveEvent* event);
    virtual void dropEvent(QDropEvent* event);

private slots:
    void handleMousePress(const QString& name, const QString& xml, const QPoint& global_mouse_pos);

private:
    // QDesignerFormEditorInterface *m_core;
    SampleDesignerInterface* m_core;

    WidgetBoxTreeWidget* m_view;
};

} // namespace qdesigner_internal

QT_END_NAMESPACE

#endif // BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOX_H
