#ifndef SPECTURM_TYPES_H
#define SPECTURM_TYPES_H

#include <vector>

namespace tspectrum {

using Int_t = int;
using Double_t = double;
using Bool_t = bool;
const bool kTRUE = true;
const bool kFALSE = false;

using vec2d = std::vector<std::vector<double>>;

};

#endif
