//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      gui2/dataloader/dataloader_constants.h
//! @brief     Defines class CLASS?
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2020
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI2_DATALOADER_DATALOADER_CONSTANTS_H
#define BORNAGAIN_GUI2_DATALOADER_DATALOADER_CONSTANTS_H

#include <string>
#include <vector>

namespace gui2::Constants {

const std::string AxisType = "Axis";
const std::string IntensityType = "Intensity";
const std::string IgnoreType = "Ignore";

} // namespace gui2::Constants

#endif // BORNAGAIN_GUI2_DATALOADER_DATALOADER_CONSTANTS_H
