//  ************************************************************************************************
//
//  qt-mvvm: Model-view-view-model framework for large GUI applications
//
//! @file      mvvm/model/mvvm/core/variant.h
//! @brief     Defines class CLASS?
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2020
//! @authors   Gennady Pospelov et al, Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_MVVM_MODEL_MVVM_CORE_VARIANT_H
#define BORNAGAIN_MVVM_MODEL_MVVM_CORE_VARIANT_H

//! @file mvvm/model/mvvm/core/variant.h
//! Defines variant type (intermediate step toward migration from QVariant to std::variant).

#include <QVariant>

using Variant = QVariant;

#endif // BORNAGAIN_MVVM_MODEL_MVVM_CORE_VARIANT_H
