/****************************************************************************
**
** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Designer of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOXTREEWIDGET_H
#define BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOXTREEWIDGET_H

#include "GUI/coregui/Views/widgetbox/qdesigner_widgetbox_p.h"

#include <QHash>
#include <QIcon>
#include <QList>
#include <QTreeWidget>
#include <QXmlStreamReader> // Cannot forward declare them on Mac
#include <QXmlStreamWriter>

QT_BEGIN_NAMESPACE

class QDesignerFormEditorInterface;
class QDesignerDnDItemInterface;

class QTimer;

class SampleDesignerInterface;

namespace qdesigner_internal {

class WidgetBoxCategoryListView;

//! WidgetBoxTreeWidget: A tree of categories
class WidgetBoxTreeWidget : public QTreeWidget {
    Q_OBJECT

public:
    typedef QDesignerWidgetBoxInterface::Widget Widget;
    typedef QDesignerWidgetBoxInterface::Category Category;
    typedef QDesignerWidgetBoxInterface::CategoryList CategoryList;

    explicit WidgetBoxTreeWidget(SampleDesignerInterface* core, QWidget* parent = 0);
    virtual ~WidgetBoxTreeWidget();

    int categoryCount() const;
    Category category(int cat_idx) const;
    void addCategory(const Category& cat);
    void removeCategory(int cat_idx);

    int widgetCount(int cat_idx) const;
    Widget widget(int cat_idx, int wgt_idx) const;
    void addWidget(int cat_idx, const Widget& wgt);
    void removeWidget(int cat_idx, int wgt_idx);

    void dropWidgets(const QList<QDesignerDnDItemInterface*>& item_list);

    void setFileName(const QString& file_name);
    QString fileName() const;
    bool load(QDesignerWidgetBox::LoadMode loadMode);
    bool loadContents(const QString& contents);
    bool save();
    QIcon iconForWidget(QString iconName) const;

signals:
    void pressed(const QString name, const QString dom_xml, const QPoint& global_mouse_pos);

public slots:
    void filter(const QString&);

protected:
    virtual void contextMenuEvent(QContextMenuEvent* e);
    virtual void resizeEvent(QResizeEvent* e);

private slots:
    void slotSave();
    void slotScratchPadItemDeleted();
    void slotLastScratchPadItemDeleted();

    void handleMousePress(QTreeWidgetItem* item);
    void deleteScratchpad();
    void slotListMode();
    void slotIconMode();

private:
    WidgetBoxCategoryListView* addCategoryView(QTreeWidgetItem* parent, bool iconMode);
    WidgetBoxCategoryListView* categoryViewAt(int idx) const;
    void adjustSubListSize(QTreeWidgetItem* cat_item);

    static bool readCategories(const QString& fileName, const QString& xml, CategoryList* cats,
                               QString* errorMessage);
    static bool readWidget(Widget* w, const QString& xml, QXmlStreamReader& r);

    CategoryList loadCustomCategoryList() const;
    void writeCategories(QXmlStreamWriter& writer, const CategoryList& cat_list) const;

    int indexOfCategory(const QString& name) const;
    int indexOfScratchpad() const;
    int ensureScratchpad();
    void addCustomCategories(bool replace);

    void saveExpandedState() const;
    void restoreExpandedState();
    void updateViewMode();

    SampleDesignerInterface* m_core;
    QString m_file_name;
    typedef QHash<QString, QIcon> IconCache;
    mutable IconCache m_pluginIcons;
    bool m_iconMode;
    QTimer* m_scratchPadDeleteTimer;
};

} // namespace qdesigner_internal

QT_END_NAMESPACE

#endif // BORNAGAIN_GUI_COREGUI_VIEWS_WIDGETBOX_WIDGETBOXTREEWIDGET_H
